@@ -3,12 +3,15 @@ use autoswap::do_autoswap;
 use colored::Colorize;
 use melwallet_client::{DaemonClient, WalletClient, WalletSummary};
 use smol::{prelude::*, process::Child};
-use std::{convert::TryInto, io::Write, time::Duration};
+use std::{io::Write, time::Duration};
 use std::{net::SocketAddr, str::FromStr};
 use structopt::StructOpt;
 use tabwriter::TabWriter;
-use themelio_stf::{melvm::Address, CoinData, CoinID, Denom, NetID, PoolKey, TxHash, TxKind};
-use tmelcrypt::{Ed25519SK, HashVal};
+use themelio_stf::{
+    melvm::{Address, Covenant},
+    CoinData, CoinID, Denom, NetID, PoolKey, StakeDoc, Transaction, TxHash, TxKind, STAKE_EPOCH,
+};
+use tmelcrypt::{Ed25519PK, HashVal};
 mod autoswap;
 
 #[derive(StructOpt, Clone, Debug)]
@@ -61,6 +64,21 @@ enum Args {
         /// How much money to swap
         value: u128,
     },
+    /// Stakes a certain number of syms.
+    Stake {
+        #[structopt(flatten)]
+        wargs: WalletArgs,
+        /// How many microsyms to stake
+        value: u128,
+        /// Ed25519 public key of the staker that receives voting rights
+        staker_pubkey: String,
+        /// When the stake takes effect. By default, as soon as possible.
+        #[structopt(long)]
+        start: Option<u64>,
+        /// How long will the stake last. By default, 1 epoch with 1 epoch waiting time.
+        #[structopt(long)]
+        duration: Option<u64>,
+    },
     /// Send a transaction to the network
     Send {
         #[structopt(flatten)]
@@ -68,9 +86,19 @@ enum Args {
         #[structopt(long)]
         /// A string specifying who to send money to, in the format "dest,amount[,denom[,additional_data]]". For example, --to $ADDRESS,1 sends 1 µMEL to $ADDRESS. Can be specified multiple times to send money to multiple addresses.
         to: Vec<CoinDataWrapper>,
+        /// Force the selection of a coin
         #[structopt(long)]
-        /// Hexadecimal secret key. Optional if the wallet is unlocked.
-        secret: Option<String>,
+        force_spend: Vec<CoinID>,
+        /// Additional covenants. This often must be specified if we are spending coins that belong to other addresses, like covenant coins.
+        #[structopt(long)]
+        add_covenant: Vec<String>,
+    },
+    /// Force-reverts a transaction. DO NOT use unless you're sure the transaction is not on the blockchain and will not be!
+    ForceRevert {
+        #[structopt(flatten)]
+        wargs: WalletArgs,
+        /// Transaction hash
+        txhash: HashVal,
     },
     /// Unlocks a wallet. Will read password from stdin.
     Unlock {
@@ -108,7 +136,7 @@ impl FromStr for CoinDataWrapper {
                 let amount: u128 = amount.parse()?;
                 Ok(CoinDataWrapper(CoinData {
                     covhash: dest,
-                    value: amount,
+                    value: amount.into(),
                     denom: Denom::Mel,
                     additional_data: vec![],
                 }))
@@ -118,7 +146,7 @@ impl FromStr for CoinDataWrapper {
                 let amount: u128 = amount.parse()?;
                 Ok(CoinDataWrapper(CoinData {
                     covhash: dest,
-                    value: amount,
+                    value: amount.into(),
                     denom: denom.parse()?,
                     additional_data: vec![],
                 }))
@@ -129,7 +157,7 @@ impl FromStr for CoinDataWrapper {
                 let additional_data: Vec<u8> = hex::decode(&additional_data)?;
                 Ok(CoinDataWrapper(CoinData {
                     covhash: dest,
-                    value: amount,
+                    value: amount.into(),
                     denom: denom.parse()?,
                     additional_data,
                 }))
@@ -256,49 +284,103 @@ fn main() -> http_types::Result<()> {
                 let summary = wargs.wallet().await?.summary().await?;
                 write_wallet_summary(&mut twriter, &wargs.wallet, &summary)?;
             }
-            Args::Send { wargs, to, secret } => {
+            Args::Send {
+                wargs,
+                to,
+                force_spend,
+                add_covenant,
+            } => {
                 let wallet = wargs.wallet().await?;
-                let secret = secret
-                    .map(|secret| Ed25519SK(hex::decode(&secret).unwrap().try_into().unwrap()));
                 let desired_outputs = to.iter().map(|v| v.0.clone()).collect::<Vec<_>>();
                 let tx = wallet
                     .prepare_transaction(
                         TxKind::Normal,
-                        vec![],
-                        desired_outputs.clone(),
-                        secret,
+                        force_spend,
+                        desired_outputs,
+                        add_covenant
+                            .into_iter()
+                            .map(|s| Ok(Covenant(hex::decode(&s)?)))
+                            .collect::<anyhow::Result<Vec<_>>>()?,
                         vec![],
                         vec![],
                     )
                     .await?;
-                writeln!(twriter, "{}", "TRANSACTION RECIPIENTS".bold())?;
-                writeln!(twriter, "{}", "Address\tAmount\tAdditional data".italic())?;
-                for output in desired_outputs {
-                    writeln!(
-                        twriter,
-                        "{}\t{} {}\t{:?}",
-                        output.covhash.to_string().bright_blue(),
-                        output.value,
-                        match output.denom {
-                            Denom::Mel => "µMEL",
-                            Denom::Sym => "µSYM",
-                            Denom::NomDosc => "µnomDOSC",
-                            Denom::Custom(_) => "(custom token)",
-                            Denom::NewCoin => "(new token type)",
-                        },
-                        hex::encode(&output.additional_data)
-                    )?;
-                }
+                send_tx(&mut twriter, stdin, wallet, tx).await?
+            }
+            Args::ForceRevert { wargs, txhash } => {
+                let wallet = wargs.wallet().await?;
+                wallet.force_revert_transaction(txhash.into()).await?;
+            }
+            Args::Stake {
+                wargs,
+                value,
+                start,
+                duration,
+                staker_pubkey,
+            } => {
+                let staker_pubkey = Ed25519PK::from_bytes(
+                    &hex::decode(&staker_pubkey).context("staker pubkey must be hex")?,
+                )
+                .context("staker pubkey must be 32 bytes")?;
+                let wallet = wargs.wallet().await?;
+                let last_header = wargs
+                    .common
+                    .dclient()
+                    .get_summary(wallet.summary().await?.network == NetID::Testnet)
+                    .await?;
+                let next_epoch = last_header.height.epoch() + 1;
+                let start_epoch = start.unwrap_or_default().max(next_epoch);
+                let duration = duration.unwrap_or(1);
+                let end_epoch = start_epoch + duration;
+                let post_end_epoch = start_epoch + duration + 1;
+                const BLOCKS_IN_DAY: u64 = 2880;
                 writeln!(
                     twriter,
-                    "{}\t{} µMEL",
-                    " (network fees)".yellow(),
-                    tx.fee.to_string()
+                    "Latest:\tblock {} (epoch {})",
+                    (last_header.height).to_string().bold(),
+                    last_header.height / STAKE_EPOCH
                 )?;
-                twriter.flush()?;
-                proceed_prompt(&mut stdin).await?;
-                let txhash = wallet.send_tx(tx).await?;
-                write_txhash(&mut twriter, &wargs.wallet, txhash)?;
+                writeln!(
+                    twriter,
+                    "Voting rights start:\tblock {} (epoch {}, in {} days)",
+                    (start_epoch * STAKE_EPOCH).to_string().bold().bright_blue(),
+                    start_epoch,
+                    (start_epoch * STAKE_EPOCH - last_header.height.0) / BLOCKS_IN_DAY
+                )?;
+                writeln!(
+                    twriter,
+                    "Voting rights end:\tblock {} (epoch {}, in {} days)",
+                    (end_epoch * STAKE_EPOCH).to_string().bold().bright_yellow(),
+                    end_epoch,
+                    (end_epoch * STAKE_EPOCH - last_header.height.0) / BLOCKS_IN_DAY
+                )?;
+                writeln!(
+                    twriter,
+                    "Syms unlock:\tblock {} (epoch {}, in {} days)",
+                    (post_end_epoch * STAKE_EPOCH)
+                        .to_string()
+                        .bold()
+                        .bright_green(),
+                    post_end_epoch,
+                    (post_end_epoch * STAKE_EPOCH - last_header.height.0) / BLOCKS_IN_DAY
+                )?;
+                writeln!(
+                    twriter,
+                    "{}",
+                    "WARNING: Syms are immediately locked no matter when voting rights start!"
+                        .bright_red()
+                        .bold()
+                        .italic()
+                )?;
+                let tx = wallet
+                    .prepare_stake_transaction(StakeDoc {
+                        e_start: start_epoch,
+                        e_post_end: post_end_epoch,
+                        syms_staked: value.into(),
+                        pubkey: staker_pubkey,
+                    })
+                    .await?;
+                send_tx(&mut twriter, stdin, wallet, tx).await?
             }
             Args::WaitConfirmation { wargs, txhash } => {
                 wait_tx(&wargs.wallet().await?, TxHash(txhash)).await?
@@ -344,7 +426,7 @@ fn main() -> http_types::Result<()> {
             Args::Autoswap { wargs, value } => {
                 let daemon = wargs.common.dclient();
                 let wallet = wargs.wallet().await?;
-                do_autoswap(daemon, wallet, value).await;
+                do_autoswap(daemon, wallet, value.into()).await;
             }
             Args::Swap {
                 wargs,
@@ -361,19 +443,19 @@ fn main() -> http_types::Result<()> {
                 } else {
                     max_value
                 };
-                let value = value.unwrap_or(max_value);
+                let value = value.unwrap_or(max_value.into());
                 let pool_key = PoolKey::new(from, to);
                 let to_send = wallet
                     .prepare_transaction(
                         TxKind::Swap,
                         vec![],
                         vec![CoinData {
-                            value,
+                            value: value.into(),
                             denom: from,
                             additional_data: vec![],
                             covhash: wallet.summary().await?.address,
                         }],
-                        None,
+                        vec![],
                         pool_key.to_bytes(),
                         vec![],
                     )
@@ -415,6 +497,43 @@ fn main() -> http_types::Result<()> {
     })
 }
 
+async fn send_tx(
+    mut twriter: impl Write,
+    mut stdin: impl AsyncRead + Unpin,
+    wallet: WalletClient,
+    tx: Transaction,
+) -> anyhow::Result<()> {
+    writeln!(twriter, "{}", "TRANSACTION RECIPIENTS".bold())?;
+    writeln!(twriter, "{}", "Address\tAmount\tAdditional data".italic())?;
+    for output in tx.outputs.iter() {
+        writeln!(
+            twriter,
+            "{}\t{} {}\t{:?}",
+            output.covhash.to_string().bright_blue(),
+            output.value,
+            match output.denom {
+                Denom::Mel => "µMEL",
+                Denom::Sym => "µSYM",
+                Denom::NomDosc => "µnomDOSC",
+                Denom::Custom(_) => "(custom token)",
+                Denom::NewCoin => "(new token type)",
+            },
+            hex::encode(&output.additional_data)
+        )?;
+    }
+    writeln!(
+        twriter,
+        "{}\t{} µMEL",
+        " (network fees)".yellow(),
+        tx.fee.to_string()
+    )?;
+    twriter.flush()?;
+    proceed_prompt(&mut stdin).await?;
+    let txhash = wallet.send_tx(tx).await?;
+    write_txhash(&mut twriter, wallet.name(), txhash)?;
+    Ok(())
+}
+
 fn write_wallet_summary(
     out: &mut impl Write,
     wallet_name: &str,
@@ -436,6 +555,13 @@ fn write_wallet_summary(
         match summary.network {
             NetID::Mainnet => "mainnet".bright_green().bold(),
             NetID::Testnet => "testnet".yellow().bold(),
+            NetID::Custom02 => todo!(),
+            NetID::Custom03 => todo!(),
+            NetID::Custom04 => todo!(),
+            NetID::Custom05 => todo!(),
+            NetID::Custom06 => todo!(),
+            NetID::Custom07 => todo!(),
+            NetID::Custom08 => todo!(),
         }
     )?;
     writeln!(
@@ -457,6 +583,11 @@ fn write_wallet_summary(
         };
         writeln!(out, "\t{}\t{}", v, denom)?;
     }
+    writeln!(
+        out,
+        "Staked:\t{}",
+        format!("{}\tµSYM", summary.staked_microsym)
+    )?;
     Ok(())
 }
 
