@@ -7,9 +7,9 @@ use std::{
 use http_types::{Body, Method, Request, Response, StatusCode, Url};
 use smol::net::TcpStream;
 use themelio_stf::{
-    CoinData, CoinID, Denom, HexBytes, PoolKey, PoolState, Transaction, TxHash, TxKind,
+    melvm::Covenant, CoinData, CoinID, Denom, Header, HexBytes, PoolKey, PoolState, StakeDoc,
+    Transaction, TxHash, TxKind,
 };
-use tmelcrypt::Ed25519SK;
 
 use crate::{structs::WalletSummary, DaemonError, TransactionStatus, WalletDump};
 
@@ -33,7 +33,7 @@ impl DaemonClient {
             .await?)
     }
 
-    /// Create a wallet
+    /// Create a walletz`   
     pub async fn dump_wallet(&self, name: &str) -> Result<Option<WalletDump>, DaemonError> {
         let mut resp = http_get(self.endpoint, &format!("wallets/{}", name)).await?;
         if resp.status() == StatusCode::NotFound {
@@ -103,6 +103,20 @@ impl DaemonClient {
         .body_json()
         .await?)
     }
+
+    /// Obtains the latest header
+    pub async fn get_summary(&self, testnet: bool) -> Result<Header, DaemonError> {
+        Ok(successful(
+            http_get(
+                self.endpoint,
+                &format!("summary?{}", if testnet { "testnet=1" } else { "" }),
+            )
+            .await?,
+        )
+        .await?
+        .body_json()
+        .await?)
+    }
 }
 
 /// An interface to a particular wallet.
@@ -191,13 +205,47 @@ impl WalletClient {
         ))
     }
 
+    /// Obtain a prepared staking transaction
+    pub async fn prepare_stake_transaction(
+        &self,
+        stake_doc: StakeDoc,
+    ) -> Result<Transaction, DaemonError> {
+        Ok(successful(
+            http_with_body(
+                self.endpoint,
+                &format!("wallets/{}/prepare-stake-tx", self.wallet_name),
+                Method::Post,
+                serde_json::to_vec(&stake_doc).unwrap(),
+            )
+            .await?,
+        )
+        .await?
+        .body_json()
+        .await?)
+    }
+
+    /// Force-reverts a tx
+    pub async fn force_revert_transaction(&self, txhash: TxHash) -> Result<(), DaemonError> {
+        successful(
+            http_with_body(
+                self.endpoint,
+                &format!("wallets/{}/transactions/{}", self.wallet_name, txhash),
+                Method::Delete,
+                "".to_string(),
+            )
+            .await?,
+        )
+        .await?;
+        Ok(())
+    }
+
     /// Obtain a prepared transaction
     pub async fn prepare_transaction(
         &self,
         kind: TxKind,
         desired_inputs: Vec<CoinID>,
         desired_outputs: Vec<CoinData>,
-        secret: Option<Ed25519SK>,
+        scripts: Vec<Covenant>,
         data: Vec<u8>,
         no_balance: Vec<Denom>,
     ) -> Result<Transaction, DaemonError> {
@@ -219,12 +267,10 @@ impl WalletClient {
             "nobalance".to_string(),
             serde_json::to_value(&no_balance).unwrap(),
         );
-        if let Some(secret) = secret {
-            adhoc.insert(
-                "signing_key".to_string(),
-                serde_json::to_value(hex::encode(&secret.0)).unwrap(),
-            );
-        }
+        adhoc.insert(
+            "scripts".to_string(),
+            serde_json::to_value(scripts).unwrap(),
+        );
         Ok(successful(
             http_with_body(
                 self.endpoint,
@@ -296,6 +342,11 @@ impl WalletClient {
         .body_json()
         .await?)
     }
+
+    /// Gets the name
+    pub fn name(&self) -> &str {
+        &self.wallet_name
+    }
 }
 
 static AUTH_TOKEN: once_cell::sync::Lazy<Option<String>> =
